@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -83,17 +85,23 @@ namespace xsimd
 
         using self_type = batch<float, 8>;
         using base_type = simd_batch<self_type>;
+        using batch_bool_type = typename base_type::batch_bool_type;
 
         batch();
         explicit batch(float f);
         batch(float f0, float f1, float f2, float f3,
               float f4, float f5, float f6, float f7);
         explicit batch(const float* src);
+
         batch(const float* src, aligned_mode);
         batch(const float* src, unaligned_mode);
+        
         batch(const __m256& rhs);
         batch& operator=(const __m256& rhs);
 
+        batch(const batch_bool_type& rhs);
+        batch& operator=(const batch_bool_type& rhs);
+        
         operator __m256() const;
 
         XSIMD_DECLARE_LOAD_STORE_ALL(float, 8)
@@ -137,6 +145,17 @@ namespace xsimd
         return *this;
     }
 
+    inline batch<float, 8>::batch(const batch_bool_type& rhs)
+        : base_type(_mm256_and_ps(rhs, batch(1.f)))
+    {
+    }
+
+    inline batch<float, 8>& batch<float, 8>::operator=(const batch_bool_type& rhs)
+    {
+        this->m_value = _mm256_and_ps(rhs, batch(1.f));
+        return *this;
+    }
+
     inline batch_bool<float, 8>::operator __m256() const
     {
         return m_value;
