@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -143,6 +145,7 @@ namespace xsimd
         using value_type = typename simd_batch_traits<X>::value_type;
         static constexpr std::size_t size = simd_batch_traits<X>::size;
         using storage_type = typename simd_batch_traits<X>::storage_type;
+        using batch_bool_type = typename simd_batch_traits<X>::batch_bool_type;
 
         using iterator = value_type*;
         using const_iterator = const value_type*;
@@ -292,11 +295,74 @@ namespace xsimd
     template <class T, std::size_t N>
     batch<T, N> operator>>(const batch<T, N>& lhs, const batch<T, N>& rhs);
 
+    /**************************
+     * batch cast functions *
+     **************************/
+
+    // Provides a static_cast from batch<T_in, N> to batch<T_out, N>
+    template <class T_in, class T_out, std::size_t N>
+    struct batch_cast_impl
+    {
+        template <std::size_t... I>
+        static inline batch<T_out, N> run_impl(const batch<T_in, N>& x, detail::index_sequence<I...>)
+        {
+            return batch<T_out, N>(static_cast<T_out>(x[I])...);
+        }
+
+    public:
+        static inline batch<T_out, N> run(const batch<T_in, N>& x)
+        {
+            return run_impl(x, detail::make_index_sequence<N>{});
+        }
+    };
+
+    template <class T, std::size_t N>
+    struct batch_cast_impl<T, T, N>
+    {
+        static inline batch<T, N> run(const batch<T, N>& x)
+        {
+            return x;
+        }
+    };
+
+    // Shorthand for defining an intrinsic-based batch_cast implementation
+    #define XSIMD_BATCH_CAST_INTRINSIC(T_IN, T_OUT, N, INTRINSIC)               \
+        template <>                                                             \
+        struct batch_cast_impl<T_IN, T_OUT, N>                                  \
+        {                                                                       \
+            static inline batch<T_OUT, N> run(const batch<T_IN, N>& x)          \
+            {                                                                   \
+                return INTRINSIC(x);                                            \
+            }                                                                   \
+        };
+
+    // Shorthand for defining an intrinsic-based batch_cast implementation that requires 2 intrinsics
+    #define XSIMD_BATCH_CAST_INTRINSIC2(T_IN, T_OUT, N, INTRINSIC1, INTRINSIC2) \
+        template <>                                                             \
+        struct batch_cast_impl<T_IN, T_OUT, N>                                  \
+        {                                                                       \
+            static inline batch<T_OUT, N> run(const batch<T_IN, N>& x)          \
+            {                                                                   \
+                return INTRINSIC2(INTRINSIC1(x));                               \
+            }                                                                   \
+        };
+
+    // Shorthand for defining an implicit batch_cast implementation
+    #define XSIMD_BATCH_CAST_IMPLICIT(T_IN, T_OUT, N)                           \
+        template <>                                                             \
+        struct batch_cast_impl<T_IN, T_OUT, N>                                  \
+        {                                                                       \
+            static inline batch<T_OUT, N> run(const batch<T_IN, N>& x)          \
+            {                                                                   \
+                return batch<T_OUT, N>(x);                                      \
+            }                                                                   \
+        };
+
     /**************************
      * bitwise cast functions *
      **************************/
 
-    // Provides a reinterpret_case from batch<T_in, N_in> to batch<T_out, N_out>
+    // Provides a reinterpret_cast from batch<T_in, N_in> to batch<T_out, N_out>
     template <class B_in, class B_out>
     struct bitwise_cast_impl;
 
@@ -967,26 +1033,26 @@ namespace xsimd
         return lhs() OP batch_type_t<X>(rhs);                                                      \
     }
 
-#define XSIMD_BINARY_BOOL_OP_DERIVED(OP, FUNC)                                                     \
+#define XSIMD_BINARY_BOOL_OP_DERIVED(OP, BASE_OP)                                                  \
     template <class X>                                                                             \
     inline typename simd_batch_traits<X>::batch_bool_type operator OP(const simd_base<X>& lhs,     \
                                                                       const simd_base<X>& rhs)     \
     {                                                                                              \
-        return FUNC;                                                                               \
+        return rhs() BASE_OP lhs();                                                                \
     }                                                                                              \
                                                                                                    \
     template <class X>                                                                             \
     inline typename simd_batch_traits<X>::batch_bool_type operator OP(                             \
         const typename simd_batch_traits<X>::value_type& lhs, const simd_base<X>& rhs)             \
     {                                                                                              \
-        return FUNC;                                                                               \
+        return rhs() BASE_OP batch_type_t<X>(lhs);                                                 \
     }                                                                                              \
                                                                                                    \
     template <class X>                                                                             \
     inline typename simd_batch_traits<X>::batch_bool_type operator OP(                             \
         const simd_base<X>& lhs, const typename simd_batch_traits<X>::value_type& rhs)             \
     {                                                                                              \
-        return FUNC;                                                                               \
+        return batch_type_t<X>(rhs) BASE_OP lhs();                                                 \
     }
 
     /**
@@ -1302,7 +1368,7 @@ namespace xsimd
     typename simd_batch_traits<X>::batch_bool_type
     operator>(const simd_base<X>& lhs, const simd_base<X>& rhs);
 
-    XSIMD_BINARY_BOOL_OP_DERIVED(>, rhs() < lhs())
+    XSIMD_BINARY_BOOL_OP_DERIVED(>, <)
 
     /**
      * @ingroup simd_batch_comparison
@@ -1317,7 +1383,7 @@ namespace xsimd
     typename simd_batch_traits<X>::batch_bool_type
     operator>=(const simd_base<X>& lhs, const simd_base<X>& rhs);
 
-    XSIMD_BINARY_BOOL_OP_DERIVED(>=, rhs() <= lhs())
+    XSIMD_BINARY_BOOL_OP_DERIVED(>=, <=)
 
     /**
      * @defgroup simd_batch_bitwise Bitwise operators
@@ -1699,6 +1765,16 @@ namespace xsimd
         GENERIC_OPERATOR_IMPLEMENTATION(>>);
     }
 
+    /*****************************************
+     * batch cast functions implementation *
+     *****************************************/
+
+    template <class T_out, class T_in, std::size_t N>
+    inline batch<T_out, N> batch_cast(const batch<T_in, N>& x)
+    {
+        return batch_cast_impl<T_in, T_out, N>::run(x);
+    }
+
     /*****************************************
      * bitwise cast functions implementation *
      *****************************************/
