@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -79,6 +81,7 @@ namespace xsimd
         static constexpr std::size_t size = 8;
         using batch_bool_type = batch_bool<int16_t, 8>;
         static constexpr std::size_t align = 16;
+        using storage_type = __m128i;
     };
 
     template <>
@@ -88,6 +91,7 @@ namespace xsimd
         static constexpr std::size_t size = 8;
         using batch_bool_type = batch_bool<uint16_t, 8>;
         static constexpr std::size_t align = 16;
+        using storage_type = __m128i;
     };
 
     template <>
@@ -102,7 +106,8 @@ namespace xsimd
         using base_class::store_aligned;
         using base_class::store_unaligned;
 
-        XSIMD_DECLARE_LOAD_STORE_INT16(int16_t, 8);
+        XSIMD_DECLARE_LOAD_STORE_INT16(int16_t, 8)
+        XSIMD_DECLARE_LOAD_STORE_LONG(int16_t, 8)
     };
 
     template <>
@@ -117,13 +122,18 @@ namespace xsimd
         using base_class::store_aligned;
         using base_class::store_unaligned;
 
-        XSIMD_DECLARE_LOAD_STORE_INT16(uint16_t, 8);
+        XSIMD_DECLARE_LOAD_STORE_INT16(uint16_t, 8)
+        XSIMD_DECLARE_LOAD_STORE_LONG(uint16_t, 8)
     };
 
     batch<int16_t, 8> operator<<(const batch<int16_t, 8>& lhs, int32_t rhs);
     batch<int16_t, 8> operator>>(const batch<int16_t, 8>& lhs, int32_t rhs);
+    batch<int16_t, 8> operator<<(const batch<int16_t, 8>& lhs, const batch<int16_t, 8>& rhs);
+    batch<int16_t, 8> operator>>(const batch<int16_t, 8>& lhs, const batch<int16_t, 8>& rhs);
     batch<uint16_t, 8> operator<<(const batch<uint16_t, 8>& lhs, int32_t rhs);
     batch<uint16_t, 8> operator>>(const batch<uint16_t, 8>& lhs, int32_t rhs);
+    batch<uint16_t, 8> operator<<(const batch<uint16_t, 8>& lhs, const batch<int16_t, 8>& rhs);
+    batch<uint16_t, 8> operator>>(const batch<uint16_t, 8>& lhs, const batch<int16_t, 8>& rhs);
 
     /************************************
      * batch<int16_t, 8> implementation *
@@ -253,27 +263,65 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT16(int16_t, 8, 16)
+    XSIMD_DEFINE_LOAD_STORE_LONG(int16_t, 8, 16)
 
     inline batch<int16_t, 8> operator<<(const batch<int16_t, 8>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](int16_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
+        return _mm_slli_epi16(lhs, rhs);
     }
 
     inline batch<int16_t, 8> operator>>(const batch<int16_t, 8>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](int16_t lhs, int32_t s) { return lhs >> s; }, lhs, rhs);
+        return _mm_srai_epi16(lhs, rhs);
+    }
+
+    inline batch<int16_t, 8> operator<<(const batch<int16_t, 8>& lhs, const batch<int16_t, 8>& rhs)
+    {
+#if defined(XSIMD_AVX512VL_AVAILABLE) && defined(XSIMD_AVX512BW_AVAILABLE)
+        return _mm_sllv_epi16(lhs, rhs);
+#else
+        return sse_detail::shift_impl([](int16_t lhs, int16_t s) { return lhs << s; }, lhs, rhs);
+#endif
+    }
+
+    inline batch<int16_t, 8> operator>>(const batch<int16_t, 8>& lhs, const batch<int16_t, 8>& rhs)
+    {
+#if defined(XSIMD_AVX512VL_AVAILABLE) && defined(XSIMD_AVX512BW_AVAILABLE)
+        return _mm_srav_epi16(lhs, rhs);
+#else
+        return sse_detail::shift_impl([](int16_t lhs, int16_t s) { return lhs >> s; }, lhs, rhs);
+#endif
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT16(uint16_t, 8, 16)
+    XSIMD_DEFINE_LOAD_STORE_LONG(uint16_t, 8, 16)
 
     inline batch<uint16_t, 8> operator<<(const batch<uint16_t, 8>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](uint16_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
+        return _mm_slli_epi16(lhs, rhs);
     }
 
     inline batch<uint16_t, 8> operator>>(const batch<uint16_t, 8>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](uint16_t lhs, int32_t s) { return lhs >> s; }, lhs, rhs);
+        return _mm_srli_epi16(lhs, rhs);
+    }
+
+    inline batch<uint16_t, 8> operator<<(const batch<uint16_t, 8>& lhs, const batch<int16_t, 8>& rhs)
+    {
+#if defined(XSIMD_AVX512VL_AVAILABLE) && defined(XSIMD_AVX512BW_AVAILABLE)
+        return _mm_sllv_epi16(lhs, rhs);
+#else
+        return sse_detail::shift_impl([](uint16_t lhs, int16_t s) { return lhs << s; }, lhs, rhs);
+#endif
+    }
+
+    inline batch<uint16_t, 8> operator>>(const batch<uint16_t, 8>& lhs, const batch<int16_t, 8>& rhs)
+    {
+#if defined(XSIMD_AVX512VL_AVAILABLE) && defined(XSIMD_AVX512BW_AVAILABLE)
+        return _mm_srlv_epi16(lhs, rhs);
+#else
+        return sse_detail::shift_impl([](uint16_t lhs, int16_t s) { return lhs >> s; }, lhs, rhs);
+#endif
     }
 }
 
