@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -79,6 +81,7 @@ namespace xsimd
         static constexpr std::size_t size = 16;
         using batch_bool_type = batch_bool<int8_t, 16>;
         static constexpr std::size_t align = 16;
+        using storage_type = __m128i;
     };
 
     template <>
@@ -88,6 +91,7 @@ namespace xsimd
         static constexpr std::size_t size = 16;
         using batch_bool_type = batch_bool<uint8_t, 16>;
         static constexpr std::size_t align = 16;
+        using storage_type = __m128i;
     };
 
     template <>
@@ -119,7 +123,8 @@ namespace xsimd
         {
         }
 
-        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 16);
+        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 16)
+        XSIMD_DECLARE_LOAD_STORE_LONG(int8_t, 16)
     };
 
     template <>
@@ -134,13 +139,18 @@ namespace xsimd
         using base_class::store_aligned;
         using base_class::store_unaligned;
 
-        XSIMD_DECLARE_LOAD_STORE_INT8(uint8_t, 16);
+        XSIMD_DECLARE_LOAD_STORE_INT8(uint8_t, 16)
+        XSIMD_DECLARE_LOAD_STORE_LONG(uint8_t, 16)
     };
 
     batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, int32_t rhs);
     batch<int8_t, 16> operator>>(const batch<int8_t, 16>& lhs, int32_t rhs);
+    batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, const batch<int8_t, 16>& rhs);
+    batch<int8_t, 16> operator>>(const batch<int8_t, 16>& lhs, const batch<int8_t, 16>& rhs);
     batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, int32_t rhs);
     batch<uint8_t, 16> operator>>(const batch<uint8_t, 16>& lhs, int32_t rhs);
+    batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, const batch<int8_t, 16>& rhs);
+    batch<uint8_t, 16> operator>>(const batch<uint8_t, 16>& lhs, const batch<int8_t, 16>& rhs);
 
     /************************************
      * batch<int8_t, 16> implementation *
@@ -225,7 +235,6 @@ namespace xsimd
 
             static batch_type max(const batch_type& lhs, const batch_type& rhs)
             {
-
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
                 return _mm_max_epi8(lhs, rhs);
 #else
@@ -238,7 +247,7 @@ namespace xsimd
             static batch_type abs(const batch_type& rhs)
             {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSSE3_VERSION
-                return _mm_sign_epi8(rhs, rhs);
+                return _mm_abs_epi8(rhs);
 #else
                 __m128i neg = _mm_sub_epi8(_mm_setzero_si128(), rhs);
                 return _mm_min_epu8(rhs, neg);
@@ -282,27 +291,52 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT8(int8_t, 16, 16)
+    XSIMD_DEFINE_LOAD_STORE_LONG(int8_t, 16, 16)
 
     inline batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](int8_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
+        return _mm_and_si128(_mm_set1_epi8(0xFF << rhs), _mm_slli_epi32(lhs, rhs));
     }
 
     inline batch<int8_t, 16> operator>>(const batch<int8_t, 16>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](int8_t lhs, int32_t s) { return lhs >> s; }, lhs, rhs);
+        __m128i sign_mask = _mm_set1_epi16((0xFF00 >> rhs) & 0x00FF);
+        __m128i cmp_is_negative = _mm_cmpgt_epi8(_mm_setzero_si128(), lhs);
+        __m128i res = _mm_srai_epi16(lhs, rhs);
+        return _mm_or_si128(_mm_and_si128(sign_mask, cmp_is_negative), _mm_andnot_si128(sign_mask, res));
+    }
+
+    inline batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, const batch<int8_t, 16>& rhs)
+    {
+        return sse_detail::shift_impl([](int8_t lhs, int8_t s) { return lhs << s; }, lhs, rhs);
+    }
+
+    inline batch<int8_t, 16> operator>>(const batch<int8_t, 16>& lhs, const batch<int8_t, 16>& rhs)
+    {
+        return sse_detail::shift_impl([](int8_t lhs, int8_t s) { return lhs >> s; }, lhs, rhs);
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT8(uint8_t, 16, 16)
+    XSIMD_DEFINE_LOAD_STORE_LONG(uint8_t, 16, 16)
 
     inline batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](uint8_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
+        return _mm_and_si128(_mm_set1_epi8(0xFF << rhs), _mm_slli_epi32(lhs, rhs));
     }
 
     inline batch<uint8_t, 16> operator>>(const batch<uint8_t, 16>& lhs, int32_t rhs)
     {
-        return sse_detail::shift_impl([](uint8_t lhs, int32_t s) { return lhs >> s; }, lhs, rhs);
+        return _mm_and_si128(_mm_set1_epi8(0xFF >> rhs), _mm_srli_epi32(lhs, rhs));
+    }
+
+    inline batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, const batch<int8_t, 16>& rhs)
+    {
+        return sse_detail::shift_impl([](uint8_t lhs, int8_t s) { return lhs << s; }, lhs, rhs);
+    }
+
+    inline batch<uint8_t, 16> operator>>(const batch<uint8_t, 16>& lhs, const batch<int8_t, 16>& rhs)
+    {
+        return sse_detail::shift_impl([](uint8_t lhs, int8_t s) { return lhs >> s; }, lhs, rhs);
     }
 }
 
